import numpy as np

import mwatershed


def test_agglom_3d():
    offsets = [(0, 0, 1), (0, 1, 0), (1, 0, 0)]
    affinities = (
        np.array(
            [
                [[[1, 0], [0, 0]], [[0, 0], [1, 0]]],
                [[[1, 0], [0, 0]], [[0, 1], [0, 0]]],
                [[[1, 0], [0, 1]], [[0, 0], [0, 0]]],
            ],
            dtype=float,
        )
        - 0.5
    )
    # 8 nodes. connecting edges:
    # 1-2, 1-3, 1-7, 4-8, 6-8, 7-8
    # components: [(1,2,3,7),(4,6,7,8)]

    components = mwatershed.agglom(affinities, offsets)

    _, counts = np.unique(components, return_counts=True)
    counts = sorted(counts)
    assert len(np.unique(components)) == 2

    assert counts[0] == 4
    assert counts[1] == 4


def test_agglom_2d():
    offsets = [(0, 1), (1, 0)]
    affinities = (
        np.array(
            [[[0, 1, 0], [0, 1, 0], [0, 1, 0]], [[0, 0, 0], [1, 1, 1], [0, 0, 0]]],
            dtype=float,
        )
        - 0.5
    )
    # 9 nodes. connecting edges:
    # 2-3, 5-6, 8-9, 4-7, 5-8, 6-9
    # components: [(1,),(2,3),(4,7),(5,6,8,9)]

    components = mwatershed.agglom(affinities, offsets)

    _, counts = np.unique(components, return_counts=True)
    counts = sorted(counts)
    assert len(counts) == 4

    assert counts[0] == 1
    assert counts[1] == 2
    assert counts[2] == 2
    assert counts[3] == 4


def test_agglom_2d_negative_offsets():
    offsets = [(0, -1), (-1, 0)]
    affinities = (
        np.array(
            [[[0, 0, 1], [0, 0, 1], [0, 0, 1]], [[0, 0, 0], [0, 0, 0], [1, 1, 1]]],
            dtype=float,
        )
        - 0.5
    )
    # 9 nodes. connecting edges:
    # 2-3, 5-6, 8-9, 4-7, 5-8, 6-9
    # components: [(1,),(2,3),(4,7),(5,6,8,9)]

    components = mwatershed.agglom(affinities, offsets)

    _, counts = np.unique(components, return_counts=True)
    counts = sorted(counts)
    assert len(counts) == 4

    assert counts[0] == 1
    assert counts[1] == 2
    assert counts[2] == 2
    assert counts[3] == 4


def test_agglom_2d_with_extra_edges():
    nodes = np.array([[1, 0, 0], [0, 0, 0], [0, 0, 9]], dtype=np.uint64)
    offsets = [(0, 1), (1, 0)]
    affinities = (
        np.array(
            [[[0, 1, 0], [0, 1, 0], [0, 1, 0]], [[0, 0, 0], [1, 1, 1], [0, 0, 0]]],
            dtype=float,
        )
        - 0.5
    )
    edges = [(True, 9, 1)]
    # 9 nodes. connecting edges:
    # 2-3, 5-6, 8-9, 4-7, 5-8, 6-9
    # components: [(1,),(2,3),(4,7),(5,6,8,9)]

    components = mwatershed.agglom(affinities, offsets, seeds=nodes, edges=edges)

    _, counts = np.unique(components, return_counts=True)
    counts = sorted(counts)
    assert len(counts) == 3

    assert counts[0] == 2
    assert counts[1] == 2
    assert counts[2] == 5



import numpy as np

import mwatershed

import time

shape = [100, 100, 100]
offsets = [(0, 0, 1), (0, 1, 0), (1, 0, 0)]
nodes = np.zeros(shape, dtype=np.uint64)
affinities = np.random.randn(3, *shape)

t1 = time.time()
components = mwatershed.agglom(affinities, offsets)
t2 = time.time()

print(f"MWATERSHED: {len(np.unique(components))} components in {t2-t1} seconds")

from affogato.segmentation.mws import compute_mws_segmentation_from_affinities

t3 = time.time()
components = compute_mws_segmentation_from_affinities(affinities, offsets, 0)
t4 = time.time()

print(f"AFFOGATO: {len(np.unique(components))} components in {t4-t3} seconds")



import numpy as np
import matplotlib.pyplot as plt
import mwatershed

# Generate an empty ndarray of labels
labels = np.zeros((100, 100))

# Add circles with different labels
labels[25:40, 25:40] = 1
labels[60:75, 60:75] = 2
labels[50:70, 10:30] = 3

affs = np.zeros((2, 100, 100))
# affs[0] = labels > 0
# affs[1] = labels > 0
affs[0, 0, 0] = 1

result = mwatershed.agglom(affs - 0.5, [[0, 1], [1, 0]]).astype(np.uint32)
print(result.min(), result.max())

fig, ax = plt.subplots(1, 3, figsize=(10, 5))
# Plot the labels using imshow
ax[0].imshow(labels)
ax[1].imshow(affs[0])
ax[2].imshow(result)
plt.show()



import numpy as np

import mwatershed


def test_agglom_2d_with_strides():
    offsets = [(0, 1), (1, 0)]
    strides = [(2, 1), (1, 2)]
    affinities = (
        np.array(
            [[[0, 1, 0], [0, 1, 0], [0, 1, 0]], [[0, 0, 0], [1, 1, 1], [0, 0, 0]]],
            dtype=float,
        )
        - 0.5
    )
    # 9 nodes. connecting edges:
    # 2-3, 5-6, 8-9, 4-7, 5-8, 6-9
    # components: [(1,),(2,3),(4,7),(5,6,8,9)]

    components = mwatershed.agglom(affinities, offsets, strides=strides)

    _, counts = np.unique(components, return_counts=True)
    counts = sorted(counts)
    assert len(counts) == 4, components

    assert counts[0] == 2
    assert counts[1] == 2
    assert counts[2] == 2
    assert counts[3] == 3



# -*- coding: utf-8 -*-
"""Top-level package for mwatershed."""

__author__ = """William Hunter Patton"""
__email__ = """pattonw@hhmi.org"""
__version__ = """0.5.1"""
__version_info__ = tuple(int(n) for n in __version__.split("."))

from .mwatershed import agglom_rs, cluster
import numpy as np
from typing import Optional


def agglom(
    affinities: np.ndarray,
    offsets: list[list[int]],
    seeds: Optional[np.ndarray] = None,
    edges: Optional[list[tuple[bool, int, int]]] = None,
    strides: Optional[list[list[int]]] = None,
):
    return agglom_rs(affinities, offsets, seeds, edges, strides)


__all__ = ["agglom", "cluster"]
